#!/usr/local/bin/perl -w
#
# Search for and display man pages in the supplied MANPATH.
# Commands whose names are composed of terms joined by underscores may
# be abbreviated as partial command terms separated by spaces.
#
# steve.ehlers@conexant.com 11/05/02

($script = $0) =~ s@.*/@@;	# Home-made basename.
($version = '$Revision: 1.8 $ ') =~ s/\$Rev.* (.*) \$ /$1/;
($date = '$Date: 2002/12/09 23:53:33 $ ') =~ s,.* (\d+/\d+/\d+) .*,$1,;

@options = ('h',
	    's',
	    'l',
	    'M=s',
	    'debug');

require 'newgetopt.pl';

$newgetopt::ignorecase = 0 if ($newgetopt::ignorecase);

&NGetOpt(@options) || die "use -h switch to display usage statement\n";

$usage=<<EofUsage;

  $script v$version $date - Lazy man command.
    Search for (and display) the man page for a command,
    given the command in abbreviated format.
  Usage: $script [options] cmd_term1 [cmd_term2] [cmd_term3] [...]
      -M    : Specify a colon-separated man pages path.
      -l    : Loose; "rep er" matches report_all_errors
      -s    : Strict; "rep er" does not match report_all_errors <default>
      -h    : Display this help message.
    E.g.: $script rep bl b
          (instead of \"man report_black_box\")
    E.g.: $script -M \$SYNOPSYS_HOME/doc/pt/man rep
          (list and choose from all PrimeTime report commands)

EofUsage

$warn_env=<<EofWarn;
    Be sure you have the \$MANPATH environment variable
    set correctly or specify an alternate man-path with -M.
EofWarn

die "$usage" if ($#ARGV < 0) || $opt_h;

if ($opt_M) {
    $man_path = $opt_M;
} elsif (defined($ENV{'MANPATH'})) {
    $man_path = $ENV{'MANPATH'};
} else {
    die "No manpath specified";
}

# Set the wild card matching pattern for loose or strict.
#
# The loose option helps find commands if you think you are looking
# for report_blah_blah_error, but cant remember the middle part --
# "$script -l rep err" will match it.  It will also match if the
# supplied command pattern is found after the start of the command.
# (E.G., "$script -l error" will match set_error and report_error
# in addition to error_status.)
#
# The strict option requires the command to start with the supplied
# command pattern and will not skip intermediate terms.  It helps zero
# in quicker if you have a good idea of what you are looking for, and
# is the default.
#
# Strict also overrides loose if both switches are supplied, so if
# you generally prefer loose, you can create an alias
# (like llman='lman -l -M $MY_MANPATH') and still use "llman -s"
# once you get close to what you\'re looking for.
#
# Loose - match any way we can.
if ($opt_l) {
    $cmd_patt = "";		# Pattern can start anywhere in command.
    $glob = '.*?';		# Match across underscores.
}
# Strict - dont match past underscores (until after the last pattern term).
if (($opt_s) || (!$glob)) {
    $cmd_patt = "^";		# Command must start with the pattern.
    $glob = '[^_]*';		# Only match up to the next cmd term.
}

# Build up the command search string, e.g "rep*_bl*_b*"
$input_cmd = $cmd_exact = shift(@ARGV); # Get 1st command term.
$cmd_patt .= "$input_cmd$glob";
foreach $cmdterm (@ARGV) {	# Handle any additional command terms.
    $input_cmd .= " $cmdterm";	# Remember terms for "no match" message.
    $cmd_exact .= "_$cmdterm";	# Used for exact match checking.
    $cmd_patt .= "_$cmdterm$glob"; # Wild card search string.
}
print "$cmd_patt\n" if $opt_debug;

# Get the list of all available man pages in $man_path.
foreach $path (split(':', $man_path)) {
    print "Looking in $path\n" if $opt_debug;
    @dirs = `ls -d $path/*man* $path/*cat* 2> /dev/null`;
    foreach $dir (@dirs) {
	chop($dir);
	if (-d $dir) {
	    print "dir is <$dir>\n" if $opt_debug;
	    push @man_list, `ls -d $dir/* 2> /dev/null`;
	} else {
	    print "Not a dir: $dir\n" if ($opt_debug);
	}
    }
}

if ($#man_list < 0) {
    print "No man files found in $man_path\n";
    die "$warn_env";
}

# Match the command search string against each man page file found.
foreach $file (@man_list) {
    chop($file);
    $file =~ s,.*/,,;		# Get file basename.
    $file =~ s,\..*$,,;		# Strip extension (.1, .n, etc.) from end.

    # If we find an exact match for, say, the "read" command, quit looking
    # and dont list all the "read_*" commands.  We can still list all the
    # read commands by "$script rea" or "$script read_".
    if ($file eq $cmd_exact) {	# Exact command match
	%cmds = ($file, 1);	# Put ONLY this command in %cmds.
	last;			# Quit looking.
    }

    # The %cmds hash keeps us from counting the same command
    # in, say, the man/cat2 and man/man2 directories as two different
    # commands.
    ++$cmds{$file} if ($file =~ /$cmd_patt/); # Command match.
}

# Figure out where we ended up.
@cmds = sort keys %cmds;	# Sort the matching commands list into @cmds.
if ($#cmds < 0) {		# No match.
    die "No man page found to match \"$input_cmd\" pattern!\n";
} elsif ($#cmds > 0) {		# Print the list of matching commands.
    print "Multiple man pages match \"$input_cmd\":\n";
    $cmd_cnt = 0;
    # Print a numbered list of all matches.
    foreach $cmd (@cmds) {
	printf "%4d $cmd\n", ++$cmd_cnt;
    }
    # Ask user to pick from the list.
    print "Enter the desired command number (<CR> to quit): ";
    chop($cmd_num = <STDIN>);
    if ($cmd_num) {		# Respond 0 or <CR> to bail out.
	$cmd_num--;
	die "Invalid response.\n" if ($cmd_num > $#cmds) || ($cmd_num < 0);
	$cmd = $cmds[$cmd_num];
	printf "Displaying man page for %s\n", $cmd if $opt_debug;
    } else {			# User abort.
	exit 2;
    }
} else {
    $cmd = shift(@cmds);	# A single command matched the pattern.
}

# Finally, display the selected man page.
system "man -M $man_path -a $cmd";

if ($opt_s || $opt_l || $opt_h) {
    # Avoids spurious warnings like
    # "Identifier "main::opt_h" used only once"
}
